
package body Sital_textio is

    constant MAX_DIGITS : integer := 20;
    subtype int_string_buf is string(1 to MAX_DIGITS);
    
    -------------------------Body Procedures --------------------
    -----Reads a string from a line without the leading white 
    -----spaces, appends spaces if line terminates before end string.
    procedure ReadString(l : inout line; S : out string) is
    	  variable c : character;
      begin
        While (L'length > 0 and 
               (L(L'Left) = ' ' or L(L'Left) = HT or L(L'Left) = vt) ) loop
		     read(l,c); 
		  end loop;
		  for i in S'range loop
		     if l'length > 0 then 
		     	  read(l,c);
				  S(i) := c;
			  else
				  S(i) := ' ';
		     end if; 
		  end loop;
      end;


    --- Skips white spaces and returns position of non-white character.
    procedure Skip_white(variable L : in LINE; pos : inout integer)
    is
    begin
      if (L'length > 0) then 
	  while pos <= L'high loop
	    case L(pos) is
		when ' ' | HT  =>
		    pos := pos + 1;
		when others =>
		    exit;
	    end case;
	  end loop;
      end if;
    end;

    ---- shrinks line up to Pos in line.
    procedure Shrink_line(L : inout LINE; pos : in integer)
    is 
	variable old_L : LINE := L;
    begin
	if pos > 1 then
	    L := new string'(old_L(pos to old_L'high));
	    Deallocate(old_L);
	end if;
    end;

--SYS1076    procedure Shrink_line(L : inout LINE; pos : in integer)
--SYS1076    is 
--SYS1076        variable old_L : LINE := L;
--SYS1076        subtype new_string is string(1 to (L'high - pos));
--SYS1076    begin
--SYS1076        if pos > 1 then
--SYS1076	    L := new new_string;
--SYS1076          for I in  1 to (old_L'high - pos) loop 
--SYS1076            L(I) := old_L(pos + I - 1);
--SYS1076          end loop ;
--SYS1076	    Deallocate(old_L);
--SYS1076        end if;
--SYS1076    end;

    ----- 
    procedure Grow_line(L : inout LINE; incr : in integer)
    is 
	variable old_L : LINE := L;
	variable bfp: integer;  -- Blank fill pointer.
    begin
	assert incr > 0
	    report "Textio: Grow_line called with zero increment."
	    severity error;

--SYS1076		if L = null then
--SYS1076 	    bfp := 0;
--SYS1076           for I in  1 to incr loop 
--SYS1076             L(I) := ' ';
--SYS1076	          end loop ;
--SYS1076 
--SYS1076 	    L := new string(1 to incr);

	if L = null then
	    bfp := 0;
	    L := new string(1 to incr);
	else
	    bfp := old_L'high;
	    L := new string(old_L'low to old_L'high + incr);
	    L(old_L'low to old_L'high) := old_L.all;
	    Deallocate(old_L);
	end if;
	for i in 1 to incr loop
	    L(bfp + i) := ' ';
	end loop;
    end;

    procedure Int_to_string(
	constant val : in integer;
	variable result:  out int_string_buf;
	variable last: out  integer)
    is
	variable buf : string(MAX_DIGITS downto 1);
	variable pos : integer := 1;
	variable tmp : integer := abs(val);
	variable digit : integer;
    begin
	loop
	    digit := abs(tmp MOD 10);    -- MOD of integer'left returns neg number!
	    tmp := tmp / 10;
	    buf(pos) := character'val(character'pos('0') + digit);
	    pos := pos + 1;
	    exit when tmp = 0;
	end loop;
	if val < 0 then
	    buf(pos) := '-';
	    pos := pos + 1;
	end if;
	pos := pos - 1;
	result(1 to pos) := buf(pos downto 1);
	last := pos;
    end Int_to_string; -- procedure

    function Int_to_string(val : integer)
	return string
    is
	variable buf : int_string_buf;
	variable last : integer;
    begin
	Int_to_string(val, buf, last);
	return buf(1 to last);
    end Int_to_string; -- function
   ----------------------------------------------------------
   function HEX_to_string(value : std_logic_vector) return string is
		variable quad: bit_vector(0 to 3);
		constant ne:   integer := value'length/4;
		variable bv:   bit_vector(0 to value'length-1) := to_bitvector(value);
		variable s:    string(1 to ne) := (others => 'X');
	begin
		if value'length mod 4 /= 0 then
			assert FALSE report 
				"HEX_to_string Error: Trying to read vector " &
				   "with an odd (non multiple of 4) length";
			return S;
		end if;

		for i in 0 to ne-1 loop
			quad := bv(4*i to 4*i+3);
			case quad is
				when x"0" => s(i+1) := '0';
				when x"1" => s(i+1) := '1';
				when x"2" => s(i+1) := '2';
				when x"3" => s(i+1) := '3';
				when x"4" => s(i+1) := '4';
				when x"5" => s(i+1) := '5';
				when x"6" => s(i+1) := '6';
				when x"7" => s(i+1) := '7';
				when x"8" => s(i+1) := '8';
				when x"9" => s(i+1) := '9';
				when x"A" => s(i+1) := 'A';
				when x"B" => s(i+1) := 'B';
				when x"C" => s(i+1) := 'C';
				when x"D" => s(i+1) := 'D';
				when x"E" => s(i+1) := 'E';
				when x"F" => s(i+1) := 'F';
			end case;
		end loop;
		return S;
		
   end HEX_to_string;

   ----------------------------------------------------------
   procedure StrStr(variable l : inout line; S : string; 
		    good :out boolean) is
      variable retval : integer := -1;
      variable pos : positive := L'left;
   begin
      ---1. skip white spaces--- 
      Skip_white(L, pos);
      Shrink_line(L,pos);
      good := true;

      if L'length >= S'length then
	 for i in S'range loop
	     if S(i) /= L(l'left+i-S'left) then
		good := false;
	     end if;
	 end loop;
      else
	 good := false;
      end if; 
   end;
      
    procedure ReadCleanLine(file f: TEXT; L : inout LINE) is
      variable pos : positive;
    begin
      readline(f,l);
      Pos := L'left;
      Skip_white(L, pos);
      Shrink_line(L,pos);
      if (l'length > 1) then 
        if l(l'left) = '-' then --- clears remark line.
         Shrink_line(L,L'right+1);
        end if;
      end if;
    end;

    procedure LineEmpty(L : inout Line;Good : out boolean) is
      variable pos : positive := L'left;
    begin
      Skip_white(L, pos);
      Shrink_line(L,pos);

      Good := (L'Length = 0);      
    end;

    procedure error_report(file f: TEXT; 
			   S : string; LineNum : in natural) is
      variable l : line;
		variable Str : string(43 to 71) := "   ERROR in test line number ";
		variable Str2: string(43 to 49) := "Time : ";
    begin
      assert false 
      report "ERROR in line : " & int_to_string(LineNum) & " - "& S  severity Error;
		write(l,str2);
		write(l,now);
		write(l,Str);
      write(l,LineNum);
      write(l,' ');
      write(l,'-');
      write(l,' ');
      write(l,S);
      writeline(f,l);
    end;

    procedure error_print(variable f: out TEXT; 
			   S : string; LineNum : in natural) is
      variable l : line;
		variable Str : string(43 to 71) := "   ERROR in test line number ";
		variable Str2: string(43 to 49) := "Time : ";
    begin
      assert false 
      report "ERROR in line : " & int_to_string(LineNum) & " - "& S  severity Error;
		write(l,str2);
		write(l,now);
		write(l,Str);
      write(l,LineNum);
      write(l,' ');
      write(l,'-');
      write(l,' ');
      write(l,S);
    end;


    procedure note_report(file f: TEXT; 
                          S : string; LineNum : in natural) is
      variable l : line;
    begin
      assert false 
      report  "Line : " & int_to_string(LineNum) & " - "& S severity note;
      write(l,S);
      writeline(f,l);
    end;

	procedure Char2QuadBits(C: Character; 
				RESULT: out Std_ulogic_VECTOR(3 downto 0);
				GOOD: out Boolean;
				ISSUE_ERROR: in Boolean) is
	begin
		case c is
			when '0' => result :=  "0000"; good := TRUE;
			when '1' => result :=  "0001"; good := TRUE;
			when '2' => result :=  "0010"; good := TRUE;
			when '3' => result :=  "0011"; good := TRUE;
			when '4' => result :=  "0100"; good := TRUE;
			when '5' => result :=  "0101"; good := TRUE;
			when '6' => result :=  "0110"; good := TRUE;
			when '7' => result :=  "0111"; good := TRUE;
			when '8' => result :=  "1000"; good := TRUE;
			when '9' => result :=  "1001"; good := TRUE;
			when 'A'|'a' => result := "1010"; good := TRUE;
			when 'B'|'b' => result := "1011"; good := TRUE;
			when 'C'|'c' => result := "1100"; good := TRUE;
			when 'D'|'d' => result := "1101"; good := TRUE;
			when 'E'|'e' => result := "1110"; good := TRUE;
			when 'F'|'f' => result := "1111"; good := TRUE;
---- extended for std_logic -------
			when 'U'|'u' => result :=  "UUUU"; good := TRUE;
			when 'X'|'x' => result :=  "XXXX"; good := TRUE;
			when 'Z'|'z' => result :=  "ZZZZ"; good := TRUE;
			when 'W'|'w' => result :=  "WWWW"; good := TRUE;
			when 'L'|'l' => result :=  "LLLL"; good := TRUE;
			when 'H'|'h' => result :=  "HHHH"; good := TRUE;
			when '-' => result :=  "----"; good := TRUE;
 
			when others =>
			   if ISSUE_ERROR then 
				   assert FALSE report
					"HREAD Error: Read a '" & c &
					   "', expected a Hex character (0-F).";
			   end if;
			   good := FALSE;
		end case;
	end;



	procedure ReadHex(L:inout LINE; 
	                  VALUE:out Std_ulogic_VECTOR;
	                  GOOD: out BOOLEAN) is
		variable ok: boolean;
		variable c:  character;
		constant ne: integer := value'length/4;
		variable bv: Std_ulogic_VECTOR(0 to value'length-1);
		variable s:  string(1 to ne-1);
	begin
		good := FALSE; -- default
		if value'length mod 4 /= 0 then
			return;
		end if;

		loop                                    -- skip white space
			read(l,c);
			exit when (((c /= ' ') and (c /= CR) and (c /= HT)) or
			(l'length = 0));
		end loop;

		Char2QuadBits(c, bv(0 to 3), ok, FALSE);
		if not ok then 
			good := FALSE;
			return;
		end if;

		read(L, s, ok);
		if not ok then
			good := FALSE;
			return;
		end if;

		for i in 1 to ne-1 loop
			Char2QuadBits(s(i), bv(4*i to 4*i+3), ok, FALSE);
			if not ok then
				good := FALSE;
				return;
			end if;
		end loop;
		good := TRUE;
		value := bv;
	end ReadHex; 

    ----Overloading of the Std_ulogic ReadHex.

	procedure ReadHex(L:inout LINE; 
	                  VALUE:out BIT_VECTOR;
	                  GOOD: out BOOLEAN) is
         variable sv: std_ulogic_vector(0 to value'length-1);
	begin
         ReadHex(L,SV,GOOD);
		value := to_bitvector(SV);
	end ReadHex; 

    ----Overloading of the Std_ulogic ReadHex.

	procedure ReadHex(L:inout LINE; 
	                  VALUE:out Std_logic_VECTOR;
	                  GOOD: out BOOLEAN) is
         variable sv: std_ulogic_vector(0 to value'length-1);
	begin
        ReadHex(L,SV,GOOD);
		value := to_stdlogicvector(SV);
	end ReadHex; 


   PROCEDURE readword(VARIABLE field_str : INOUT STRING ; work_line : INOUT LINE ; VARIABLE cursor : INOUT INTEGER) IS
	VARIABLE i :INTEGER :=1;
   BEGIN
--    procedure Skip_white(variable L : in LINE; pos : inout integer)

    Skip_white(work_line,cursor);
	WHILE ((cursor <= work_line'high) AND (work_line(cursor) /= ' ') AND (work_line(cursor) /=  HT)) LOOP 
		field_str(i) := work_line(cursor);
		cursor := cursor + 1;
		i := i+1;
	END LOOP;
    END;

FUNCTION  To_Lower  ( CONSTANT  c    : IN CHARACTER
                    ) RETURN CHARACTER IS
    VARIABLE result : CHARACTER := c;
 BEGIN
     IF ( c >= 'A' and c <= 'Z') THEN 
         result := CHARACTER'VAL( CHARACTER'POS(c) 
                                  - CHARACTER'POS('A')
                                  + CHARACTER'POS('a') );
     END IF; 
     RETURN  result; 
END To_Lower; 


    function vec2hexstring(VALUE:in std_logic_vector) return string is
    	variable quad: std_logic_vector(0 to 3);
    	constant ne:   integer := value'length/4;
    	variable bv:   std_logic_vector(0 to value'length-1) := value;
    	variable s:    string(1 to ne);
    begin
    	if value'length mod 4 /= 0 then
    		assert FALSE report 
    			"Vec2HexString Error: Trying to convert vector " &
    			   "with an odd (non multiple of 4) length";
    		return "xx";
    	end if;

    	for i in 0 to ne-1 loop
    		quad := bv(4*i to 4*i+3);
    		case quad is
    			when x"0" => s(i+1) := '0';
    			when x"1" => s(i+1) := '1';
    			when x"2" => s(i+1) := '2';
    			when x"3" => s(i+1) := '3';
    			when x"4" => s(i+1) := '4';
    			when x"5" => s(i+1) := '5';
    			when x"6" => s(i+1) := '6';
    			when x"7" => s(i+1) := '7';
    			when x"8" => s(i+1) := '8';
    			when x"9" => s(i+1) := '9';
    			when x"A" => s(i+1) := 'A';
    			when x"B" => s(i+1) := 'B';
    			when x"C" => s(i+1) := 'C';
    			when x"D" => s(i+1) := 'D';
    			when x"E" => s(i+1) := 'E';
    			when x"F" => s(i+1) := 'F';
				when "ZZZZ" => s(i+1) := 'Z';
				when "UUUU" => s(i+1) := 'U';
			    when "XXXX" => s(i+1) := 'X';
			    when "HHHH" => s(i+1) := 'H';			    
			    when "LLLL" => s(i+1) := 'L';			    	
			    when "----" => s(i+1) := '-';			    	
				when others => s(i+1) := 'x';
    		end case;
    	end loop;
		return s;
    end vec2hexstring; 

end;

