LIBRARY ieee;
USE ieee.std_logic_1164.all;

ENTITY XCVR_Both IS
   GENERIC( 
      enable_echo : boolean := true
   );
   PORT( 
      Regular  : IN     STD_LOGIC  := '1';
      TxSital  : IN     STD_LOGIC  := '0';
      TxNSital : IN     STD_LOGIC  := '0';
      EN       : IN     STD_LOGIC  := '0';
      RxSital  : OUT    STD_LOGIC;
      RxNSital : OUT    STD_LOGIC;
      Mux      : INOUT  STD_LOGIC  := 'Z'
   );

-- Declarations

END XCVR_Both ;


ARCHITECTURE SITAL_Technology OF XCVR_Both IS

    SIGNAL  transmitting : boolean;
    signal  mux_delayed : std_logic := 'Z';
    
    SIGNAL  TxDELAY             : time      := 200 ns;
    SIGNAL  RxDELAY             : time      := 200 ns;
    SIGNAL  Inhibit_value       : std_logic := '1';
    SIGNAL  RX_LINES_IDLE_VALUE : std_logic := '0';
    
    SIGNAL  EN_delayed          : STD_LOGIC;
    SIGNAL  TxSital_delayed     : STD_LOGIC;
    SIGNAL  TxNSital_delayed    : STD_LOGIC;
    

BEGIN

    TxDELAY             <= 200 ns WHEN Regular='1' ELSE 40 ns;
    RxDELAY             <= 200 ns WHEN Regular='1' ELSE 40 ns;
    Inhibit_value       <= '1'    WHEN Regular='1' ELSE '0';
    RX_LINES_IDLE_VALUE <= '0'    WHEN Regular='1' ELSE '1';
    
    EN_delayed <= transport EN after TxDELAY;
    TxSital_delayed  <= transport TxSital after TxDELAY;
    TxNSital_delayed  <= transport TxNSital after TxDELAY;
                                                       
        ---echo cancelling signal
        PROCESS (TxSital, TxNSital)
        BEGIN
            IF (TxSital'event and TxSital = '1') or (TxNSital'event and TxNSital = '1') then 
                transmitting <= true;
            ELSIF (TxSital'event or TxNSital'event) and (TxSital = '0' and TxNSital = '0') THEN 
                transmitting <= false after 500 ns;
            END IF;
        END PROCESS ;    
        mux_delayed <= transport mux after RxDELAY;
        ---reception
        PROCESS(transmitting, mux_delayed, RX_LINES_IDLE_VALUE)
        BEGIN
          IF RX_LINES_IDLE_VALUE='0' THEN
            IF enable_echo THEN
                IF mux_delayed = '1' or mux_delayed = 'H' then 
                    RxSital <= '1';
                ELSE
                    RxSital <= '0';
                END IF;
                IF mux_delayed = '0' or mux_delayed = 'L' then 
                    RxNSital <= '1';
                ELSE
                    RxNSital <= '0';
                END IF;
            ELSE -- only when not transmitting
                IF not transmitting then 
                    IF mux_delayed = '1' then 
                        RxSital <= '1';
                    ELSE
                        RxSital <= '0';
                    END IF;
                    IF mux_delayed = '0' then 
                        RxNSital <= '1';
                    ELSE
                        RxNSital <= '0';
                    END IF;
                END IF;
            END IF;
          ------------------------------------------------------------
          ELSE --idle value is '1'
            IF enable_echo THEN
                IF mux_delayed = '1' or mux_delayed = 'H' then 
                    RxNSital <= '0';
                ELSE
                    RxNSital <= '1';
                END IF;
                IF mux_delayed = '0' or mux_delayed = 'L' then 
                    RxSital <= '0';
                ELSE
                    RxSital <= '1';
                END IF;
            ELSE -- only when not transmitting
                IF not transmitting then 
                    IF mux_delayed = '1' then 
                        RxNSital <= '0';
                    ELSE
                        RxNSital <= '1';
                    END IF;
                    IF mux_delayed = '0' then 
                        RxSital <= '0';
                    ELSE
                        RxSital <= '1';
                    END IF;
                END IF;
            END IF;
          END IF;
        END PROCESS ;  

          
        ---Transmission
        mux <= 'Z' when EN_delayed = Inhibit_value ELSE 
               'Z' when TxSital_delayed = TxNSital_delayed else
               '1' when TxSital_delayed = '1' else 
               '0' when TxNSital_delayed = '1' else 
               'Z';
               
END;




