LIBRARY ieee ;
USE ieee.std_logic_1164.all;

ENTITY receiver_simple IS
   PORT( 
      Line_P     : IN     STD_LOGIC;
      Line_N     : IN     STD_LOGIC;
      receiveing : OUT    BOOLEAN;
      RxRdy      : OUT    BOOLEAN;
      Sync       : OUT    STD_LOGIC  := '0';
      Data       : OUT    STD_LOGIC_VECTOR (15 DOWNTO 0);  --MSB sent first.
      CS         : OUT    boolean;
      Gap        : OUT    time       := 4 us
   );

-- Declarations

END receiver_simple ;



ARCHITECTURE SITAL_Technology OF receiver_simple IS

    --The possible gap between line_p and line_n. above this tolerance will 
    --report a dropout error.
    constant tolerance : time := 14 ns;

    SIGNAL  n_XOR_p : boolean;
    SIGNAL  Sample_lines : boolean;
    
    SIGNAL  phase : natural;

    SIGNAL  int_Sync  : STD_LOGIC;
    SIGNAL  int_Data      : STD_LOGIC_VECTOR(15 downto 0); --MSB sent first.
              
BEGIN

    receiveing <= true when 
        (   line_n or 
            line_p or 
            line_n'delayed(200 ns) or 
            line_p'delayed(200 ns) ) = '1'
           else false;

    CS <= true;


---algorithm:
---1. Find the message enable : n_XOR_p. Will turn on/off 250 ns after bus.
---     Also check that the two lines are opposite for dropout.
---2. As long as the n_XOR_p is true check every 1/2 us the bus value and check
---     manchester code, sync type, parity.
---3. Check that the pulse widths are either 1/2, 1, 1 1/2, or 2 us in width +- tolerance.
---4. output signals to the caller. Double buffer results of this run.

--details :

---1. Find the message enable : n_XOR_p. Will turn on/off 250 ns after bus.
---     Also check that the two lines are opposite for dropout.

    n_XOR_p <= (line_n /= line_p) or 
               (line_n'delayed(tolerance) /= line_p) or 
               (line_n /= line_p'delayed(tolerance));

    PROCESS 
    BEGIN
        Sample_lines <= false;
        phase <= 1;
        wait until n_XOR_p'event and n_XOR_p;
        wait for 250 ns;
        while n_XOR_p loop
            Sample_lines <= true;
            wait for 50 ns;
            Sample_lines <= false;
            wait for 450 ns;
            IF (phase = 40) THEN
                phase <= 1;
            ELSE
                phase <= phase + 1;
            END IF;
        end loop;
    END PROCESS ;    

    PROCESS 
        VARIABLE    parity : Std_logic;
    BEGIN
        wait until Sample_lines'event and Sample_lines;
        case phase is
        --check 6 samples for sync
        when 1 =>       Int_Sync <= line_p;
                        parity := '1';
            --- GAP since the last phase of the last word !!!
    	                Gap <= phase'delayed'last_event;
        --check 32 samples for data
        when 7 to 38 => IF (phase mod 2)=1 then --first half
                            int_data(15 - (phase-7)/2) <= line_p;
                            parity := parity xor line_p; 
                         END IF;
        when OTHERS => null;
        END case ;    
    END PROCESS ;    

    PROCESS 
    BEGIN
        Wait until phase'event and phase = 40;
        wait for 250 ns;
        Sync              <= int_Sync;
        data              <= INT_DATA;
        wait for 0 ns;
        RxRdy <= true, false after 10 us; 
    END PROCESS ;    



END;



