LIBRARY ieee;
USE ieee.std_logic_1164.all;

ENTITY Transmitter IS
   PORT( 
      Go           : IN     boolean;
      Sync         : IN     std_logic;
      Data         : IN     STD_LOGIC_VECTOR (15 DOWNTO 0);  --MSB sent first.
      Line_P       : OUT    STD_LOGIC  := '0';
      Line_N       : OUT    STD_LOGIC  := '0';
      Transmitting : BUFFER boolean
   );

-- Declarations

END Transmitter ;


ARCHITECTURE SITAL_Technology OF Transmitter IS

	-- added a fifo of depth 1 that allows dual go commands and 
	--   a back to back transmission result.
	    signal  fifo_Sync      : std_logic := '0';
	    signal  fifo_Data      : STD_LOGIC_VECTOR(15 downto 0);
        shared  variable fifo_full : boolean;


BEGIN

	PROCESS(go)
	BEGIN
	    IF (go'EVENT AND go and Transmitting) THEN 
           fifo_Sync <= Sync;
           fifo_Data <= Data;
           fifo_full := true;
        end if;
	END PROCESS ;    

    PROCESS 
        VARIABLE    parity : Std_logic;
        --these will be latched upun Go'event and transmitted.
	    VARIABLE  int_Sync      : std_logic := '0';
	    VARIABLE  int_Data      : STD_LOGIC_VECTOR(15 downto 0);
    BEGIN
        Line_P <= '0'; -- after 10 ns;
        Line_N <= '0'; -- after 10 ns;
        IF fifo_full then 
			int_Sync  := fifo_Sync; 
			int_Data  := fifo_Data; 
            fifo_full := false;
        ELSE
            WAIT UNTIL (Go'EVENT AND Go);
			int_Sync := Sync; 
			int_Data := Data; 
        END IF;
        --sync
            Transmitting <= true;
            Line_P <= int_sync;
            Line_N <= not int_sync;
            WAIT FOR 1.5 us;
            Line_P <= not int_sync;
            Line_N <= int_sync;
            WAIT FOR 1.5 us;
        --16 bits
            parity := '1';
            For i in int_data'range loop
	                Line_P <= int_data(i);
	                Line_N <= not int_data(i);
	                WAIT FOR 0.5 us;
	                Line_P <= not int_data(i);
	                Line_N <= int_data(i);
	                WAIT FOR 0.5 us;
	                parity := parity xor int_data(i);
            end loop;
        --parity
            Line_P <= parity;
            Line_N <= not parity;
            WAIT FOR 0.5 us;
            Line_P <= not parity;
            Line_N <= parity;
            WAIT FOR 0.5 us;
            Transmitting <= false;
    END PROCESS ;    

END;



